/**
 * Copyright (c) 2003, David J. Yazel
 *
 * The Magicosm Development Group is granted the rights to
 * use this code in the development of the Magicosm game and all
 * subsequent derrivations in perpetuity.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) A
 * RISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE
 *
 */
package com.magicosm.skins;

import com.magicosm.skins.renderers.BaseDef;
import com.magicosm.skins.renderers.MaskedBoxDef;
import com.magicosm.skins.styles.BaseStyle;
import com.magicosm.skins.styles.ControlStyle;

import com.xith3d.utility.logs.Log;

import java.awt.*;
import java.awt.image.BufferedImage;

import java.io.IOException;

import java.net.URL;

import java.util.HashMap;
import java.util.Iterator;

import javax.imageio.ImageIO;


/**
 * Insert package comments here
 * <p>
 * Originally Coded by David Yazel on Oct 6, 2003 at 7:50:20 PM.
 */
public class Skin {
    String name;
    String author;
    String version;
    String location;
    HashMap images = new HashMap();
    HashMap fonts = new HashMap();
    HashMap renderers = new HashMap();
    HashMap styles = new HashMap();
    HashMap themes = new HashMap();

    public Skin(String location) {
        this.location = location;
    }

    private URL getUrl(String resourceName) throws IOException {
        ClassLoader classloader = ClassLoader.getSystemClassLoader();
        URL url = classloader.getResource(location + resourceName);

        if (url == null) {
            throw new IOException("Cannot find resource: " + location +
                resourceName);
        }

        return url;
    }

    public void addRenderer(SkinNodeInterface r) throws IOException {
        r.validateAndLoad(this);
        renderers.put(r.getName(), r);
    }

    public void addStyle(SkinNodeInterface r) throws IOException {
        r.validateAndLoad(this);
        styles.put(r.getName(), r);
    }

    public void addTheme(SkinNodeInterface r) throws IOException {
        r.validateAndLoad(this);
        themes.put(r.getName(), r);
    }

    public Theme getTheme(String name) {
        return (Theme) themes.get(name);
    }

    public BufferedImage getImage(String name) {
        return (BufferedImage) images.get(name);
    }

    public BaseDef getRenderer(String name) {
        return (BaseDef) renderers.get(name);
    }

    public Font getFont(String name) {
        return (Font) fonts.get(name);
    }

    public BaseStyle getStyle(String name) {
        return (BaseStyle) styles.get(name);
    }

    public void loadImage(String imageName) throws IOException {
        if (images.get(imageName) == null) {
            URL url = getUrl(imageName);
            images.put(imageName, ImageIO.read(url));
        }
    }

    public void loadFont(String fontName, String fontFileName, int style,
        int size) throws IOException {
        if (fonts.get(fontName) == null) {
            URL url = getUrl(fontFileName);

            if (url == null) {
                throw new IOException("Cannot find file " + name +
                    " on classpath");
            }

            Font font = null;

            try {
                font = Font.createFont(Font.TRUETYPE_FONT, url.openStream());
            } catch (FontFormatException e) {
                throw new IOException("cannot load font : " + fontFileName);
            }

            Font scaledFont = font.deriveFont(style, size);
            fonts.put(fontName, scaledFont);
        }
    }

    public void loadResources() throws IOException {
        Iterator i = renderers.values().iterator();

        while (i.hasNext()) {
            SkinNodeInterface r = (SkinNodeInterface) i.next();

            if (r instanceof MaskedBoxDef) {
                MaskedBoxDef box = (MaskedBoxDef) r;

                if (box.getBorderFile() != null) {
                    loadImage(box.getBorderFile());
                }

                if (box.getBorderMaskFile() != null) {
                    loadImage(box.getBorderMaskFile());
                }
            }
        }
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getAuthor() {
        return author;
    }

    public void setAuthor(String author) {
        this.author = author;
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    private boolean setThemeSuperclass(Component c, Class cClass, Theme theme) {
        if (cClass == null) {
            return false;
        }

        // recurively find the theme to match the class.
        if (c instanceof SkinComponentInterface) {
            Log.print("attmpting to get style for " + cClass.getName() +
                " in theme " + theme.getName());

            BaseStyle style = (BaseStyle) theme.map.get(cClass.getName());

            if (style != null) {
                Log.print("mapped " + c.getClass().getName() + " to " +
                    style.getClass().getName());
                ((SkinComponentInterface) c).setStyle(this, style);

                return true;
            } else if (cClass.getSuperclass() != null) {
                return setThemeSuperclass(c, cClass.getSuperclass(), theme);
            }
        } else if (cClass.getSuperclass() != null) {
            return setTheme(c, cClass.getSuperclass(), theme);
        }

        return false;
    }

    private boolean setTheme(Component c, Class cClass, Theme theme) {
        if (cClass == null) {
            return false;
        }

        // recurively find the theme to match the class.
        if (c instanceof SkinComponentInterface) {
            //            System.out.println("attmpting to get style for "+cClass.getName()+" in theme "+theme.getName());
            BaseStyle style = (BaseStyle) theme.map.get(cClass.getName());

            if (style != null) {
                System.out.println("mapped " + c.getClass().getName() + " to " +
                    style.getClass().getName());
                ((SkinComponentInterface) c).setStyle(this, style);

                return true;
            } else if (cClass.getSuperclass() != null) {
                if (setThemeSuperclass(c, cClass.getSuperclass(), theme)) {
                    return true;
                }
            }

            if (theme.extendsTheme != null) {
                return setTheme(c, cClass, theme.extendsTheme);
            }
        }

        return false;
    }

    public void setTheme(Component c, Theme theme) {
        if (theme == null) {
            throw new Error("no valid theme specified");
        }

        setTheme(c, c.getClass(), theme);

        // now handle the children
        if (c instanceof Container) {
            Component[] cs = ((Container) c).getComponents();

            for (int i = 0; i < cs.length; i++) {
                setTheme(cs[i], theme);
            }
        }
    }
}
